@@ -95,12 +95,18 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
                 assert all(
                     isinstance(e, RecordType) for e in ann_types
                 ), "Union must combine multiple PlutusData classes"
+                assert distinct(
+                    [e.record.constructor for e in ann_types]
+                ), "Union must combine PlutusData classes with unique constructors"
                 return UnionType(FrozenFrozenList(ann_types))
             if ann.value.id == "List":
                 ann_type = self.type_from_annotation(ann.slice.value)
                 assert isinstance(
                     ann_type, ClassType
                 ), "List must have a single type as parameter"
+                assert not isinstance(
+                    ann_type, TupleType
+                ), "List can currently not hold tuples"
                 return ListType(InstanceType(ann_type))
             if ann.value.id == "Dict":
                 assert isinstance(
@@ -113,7 +119,19 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
                 assert all(
                     isinstance(e, ClassType) for e in ann_types
                 ), "Dict must combine two classes"
+                assert not any(
+                    isinstance(e, TupleType) for e in ann_types
+                ), "Dict can currently not hold tuples"
                 return DictType(*(InstanceType(a) for a in ann_types))
+            if ann.value.id == "Tuple":
+                assert isinstance(
+                    ann.slice.value, Tuple
+                ), "Tuple must combine several classes"
+                ann_types = [self.type_from_annotation(e) for e in ann.slice.value.elts]
+                assert all(
+                    isinstance(e, ClassType) for e in ann_types
+                ), "Tuple must combine classes"
+                return TupleType(FrozenFrozenList([InstanceType(a) for a in ann_types]))
             raise NotImplementedError(
                 "Only Union, Dict and List are allowed as Generic types"
             )
@@ -340,7 +358,7 @@ class AggressiveTypeInferencer(CompilingNodeTransformer):
             ), f"Function '{node.name}' has no return statement but is supposed to return not-None value"
         else:
             assert (
-                functyp.rettyp == tfd.body[-1].typ
+                functyp.rettyp >= tfd.body[-1].typ
             ), f"Function '{node.name}' annotated return type does not match actual return type"
         self.exit_scope()
         # We need the function type outside for usage
@@ -616,19 +634,22 @@ class RecordReader(NodeVisitor):
         assert isinstance(
             node.target, Name
         ), "Record elements must have named attributes"
+        typ = self._type_inferencer.type_from_annotation(node.annotation)
         if node.target.id != "CONSTR_ID":
             assert (
                 node.value is None
             ), f"PlutusData attribute {node.target.id} may not have a default value"
+            assert not isinstance(
+                typ, TupleType
+            ), "Records can currently not hold tuples"
             self.attributes.append(
                 (
                     node.target.id,
-                    InstanceType(
-                        self._type_inferencer.type_from_annotation(node.annotation)
-                    ),
+                    InstanceType(typ),
                 )
             )
             return
+        assert typ == IntegerType, "CONSTR_ID must be assigned an integer"
         assert isinstance(
             node.value, Constant
         ), "CONSTR_ID must be assigned a constant integer"
